%---------------------------------------------------%
% Foot Motion Problem:                              %
%---------------------------------------------------%

clear all; close all;

handLoadControls = true;
springPosParams = false;
% addpath('C:\Files\GPOPS-II New\nlp\snopt')

% Load model parameters determined by past calibration runs
cdorig = cd;
cd('../SetupFiles')
auxdata = SetupOpenSimOptimalTracking(1,true);
cd(cdorig)

auxdata.handLoadControls = handLoadControls;
auxdata.springPosParams = springPosParams;

% Load initial and final model pose and time
t0 = auxdata.time(1);
tf = auxdata.time(end);
time = auxdata.time;
q0 = auxdata.q(1,:);
qp0 = auxdata.qp(1,:);
qf = auxdata.q(end,:);
qpf = auxdata.qp(end,:);

% Set initial guesses for Loads (control) and joint angle trajectories
% (states)
loadsGuess = auxdata.idData;
qGuess = auxdata.q;
qpGuess = auxdata.qp;
qppGuess = auxdata.qpp;
qpppGuess = auxdata.qppp;
nJerkControls = size(qpppGuess,2);
auxdata.nJerkControls = nJerkControls;

% Determine the range of the states
qRange = max(qGuess) - min(qGuess);
qpRange = max(qpGuess) - min(qpGuess);
qppRange = max(qppGuess) - min(qppGuess);
qpppRange = max(qpppGuess) - min(qpppGuess);

auxdata.activeDofs = logical(1-(qRange == 0));
auxdata.activeDofs(1:6) = [];

% Give locked DOFs a range of motion
qpppRange(qRange == 0) = 40000*pi/180;
qppRange(qRange == 0) = 4000*pi/180;
qpRange(qRange == 0) = 400*pi/180;
qRange(qRange == 0) = 60*pi/180;

% Set bounds on the states based on the previously calcualted range
qMin = min(qGuess) - 2*qRange;  
qMax  = max(qGuess) + 2*qRange;
qpMin = min(qpGuess) - 1.5*qpRange; 
qpMax  = max(qpGuess) + 1.5*qpRange;
qppMin = min(qppGuess) - 1*qppRange; 
qppMax  = max(qppGuess) + 1*qppRange;
qpppMin = min(qpppGuess) - 1*qpppRange; % 0.5 %NOTE (Nathan): This is different than previous step (ground contact calibration)
qpppMax  = max(qpppGuess) + 1*qpppRange; %0.5

% Determing the range of the controls
loadsRange = max(auxdata.idData) - min(auxdata.idData);
torquesRange = max(auxdata.torques) - min(auxdata.torques);

% Give bigger range to DOFs with small loads (for instance the toes)
loadsRange(loadsRange<1/(auxdata.tScale^2)) = 25/(auxdata.tScale^2);
torquesRange(torquesRange<1/(auxdata.tScale^2)) = 25/(auxdata.tScale^2);

% Set bounds on the torques (for when using extra controls)
loadsMin = min(auxdata.idData)-2*loadsRange; 
loadsMax = max(auxdata.idData)+2*loadsRange;

torquesMin = min(auxdata.torques) - 2*torquesRange;
torquesMax = max(auxdata.torques) + 2*torquesRange;

torquesGuess = auxdata.torques;

% Set weights for marker errors (the model tracks markers from experimental
% data
% MkrNamesOSIM = {'R.Shoulder' 'L.Shoulder' 'Neck' 'Chest' 'R.Elbow' 'R.Wrist' 'L.Elbow' 'L.Wrist' 'R.ASIS' 'L.ASIS'...
%     'Sacral' 'R.Thigh.Superior' 'R.Thigh.Inferior' 'R.Thigh.Lateral' 'R.Shank.Superior' 'R.Shank.Inferior' 'R.Shank.Lateral'...
%     'R.Heel' 'R.Midfoot.Superior' 'R.Midfoot.Lateral' 'R.Toe' 'L.Thigh.Superior' 'L.Thigh.Inferior' 'L.Thigh.Lateral'...
%     'L.Shank.Superior' 'L.Shank.Lateral' 'L.Shank.Inferior' 'L.Heel' 'L.Midfoot.Superior' 'L.Midfoot.Lateral' 'L.Toe'...
%     'L.PSIS' 'R.PSIS'};
auxdata.errorWeights = ones(1,numel(auxdata.mkrNames));
auxdata.errorWeights([18 21 28 31]) = 10;
auxdata.errorWeights([3:8]) = .1;

auxdata.errorWeights = repmat(auxdata.errorWeights,3,1);
auxdata.errorWeights = auxdata.errorWeights(:)';

% Initilize osim models in mex files
opensimPointKin_v3('Subject1_v4pt3_GPOPS.osim');
opensimID_OMP('Subject1_v4pt3_GPOPS.osim');
opensimGetBodyPosVel_OMP('Subject1_v4pt3_GPOPS.osim');

% Get median hand positions
handForceLoc = [0.01,-0.06,0; 0.01,-0.06,0]; % NOTE (Nathan): Make sure that this is the same location as where the force is applied in the model
auxdata.handForceLoc = handForceLoc;
handBodyMat = auxdata.handBodyMat;

auxdata.posOnBarR = [0.874274080385552,1.12880647759450,0.602273837458135]; %NOTE (Nathan): These come from the getWristMarkerLocs program in SetupFiles
auxdata.posOnBarL = [0.869472149835813,1.13141862779562,0.164197648932865];

% [handPos, handVel, handAng, handAngVel] = opensimGetBodyPosVel_OMP(time,[qGuess qpGuess],handForceLoc,handBodyMat);


% Min and Max hand loads allowed
% NOTE (Nathan): Thses are only used if the hand loads are minimized or for
% hand load controls
handLoadMin = -[200*ones(1,3),50*ones(1,3),200*ones(1,3),50*ones(1,3)];
handLoadMax = [200*ones(1,3),50*ones(1,3),200*ones(1,3),50*ones(1,3)];
handLoadRange = handLoadMax - handLoadMin;


% % Marker Error
% % NOTE (Nathan): I don't think these do anything
% mkrErrorsAllowed = 2e-2*ones(1,numel(auxdata.mkrNames)); %Andrew had 1e-2
% mkrErrorsAllowed([18:21 28:31]) = 1.5e-2; %Andrew had 2.5e-3
% mkrErrorsAllowed([3:8]) = 3e-2; %Andrew had 1.5e-2
% mkrErrorsAllowed = repmat(mkrErrorsAllowed, 3,1);% Duplicate for each dimension
% mkrErrorsAllowed = mkrErrorsAllowed(:)';

% GRF Errors
grfErrorsInt = 1e2*[1e1 2e1 1e1 1e1 2e1 1e1 5*ones(1,3) 5*ones(1,3)]; %5e-1* ... 5 %NOTE (Nathan): The torques are different than previous steps %Used for setting path constraint on max error of a GRF
grfErrorsMax = 3*grfErrorsInt; % 2 works better than 3 some times %Used for setting path constraint on max error of a GRF

% Coordinate Errors
qErrorsInt = 5e1*(pi/180)*ones(1,31);
% qErrorsInt(1:3) = 1*qRange(1:3); %1*
qErrorsInt(4:6) = 0.1; %100
% qErrorsInt([9 13 16 20:31]) = 0.25*qRange([9 13 16 20:31]); %0.25*
% qErrorsInt([9 13 16 20]) = 0.25*qRange([9 13 16 20]); %0.25*
% qErrorsInt(21:31) = 0.5*qRange(21:31);


qErrorsMax = 5*qErrorsInt; %5
qErrorsMax([13,20,21:31]) = (1*pi/180)*ones(1,13);

propulsiveForceIntMin = -200;
propulsiveForceIntMax = 200;

% Neural Command Errors
ncErrorMax = 0.0025*ones(1,10);

% Belt Speed and Number of Springs
auxdata.beltSpeed = 0.6;
auxdata.numSprings = sum(auxdata.numSpringsBody);


% Setup scale factors
auxdata.tMin = t0;
auxdata.tMax = tf;
auxdata.stateMin = [qMin,qpMin,qppMin];
auxdata.stateMax = [qMax,qpMax,qppMax];
auxdata.controlMin = [qpppMin,zeros(1,10)];
auxdata.controlMax = [qpppMax,1*ones(1,10)];

% auxdata.integralMin = [0*ones(1,31+6+6+12+10+31),propulsiveForceIntMin*ones(1,2)];
% auxdata.integralMax = [auxdata.tMax*([qErrorsInt ...
%     grfErrorsInt...
%     1*loadsRange([7:10,12:13,15:18,20:21])... % [7:10,12:13,15:18,20:21,23:29,33:36] Ignore pelvis, toes, wrist, patella loads. OLD: [1:6,8:13,15:end]
%     0.5*ones(1,10)... % For minimizing synergy commands (in case of indeterminicy or something)
%     2*qpppRange]).^2 ...
%     propulsiveForceIntMax*ones(1,2)]; 
auxdata.integralMin = [0*ones(1,31),propulsiveForceIntMin*ones(1,2)];
auxdata.integralMax = [auxdata.tMax*(2*qpppRange).^2 ...
    propulsiveForceIntMax*ones(1,2)]; 

onescol = ones(length(auxdata.time),1);
numTorques = size(auxdata.torques,2);
pathMult = 1e0;
auxdata.pathMin = [pathMult*[-1e-2*ones(1,3) -1e0*ones(1,3) -1e-1*ones(1,10)], ...
    -qErrorsMax -grfErrorsMax -0.005*ones(1,6) -ncErrorMax];
auxdata.pathMax = [pathMult*[1e-2*ones(1,3) 1e0*ones(1,3) 1e-1*ones(1,10)], ...
    qErrorsMax grfErrorsMax 0.005*ones(1,6) ncErrorMax];

periodicityMult = 1e0;
if handLoadControls
    auxdata.eventMin = [periodicityMult*[zeros(1,31)-0.5*pi/180 zeros(1,31)-1*pi/180 -2*[10 25 10 10 25 10]] pathMult*[-1e-1*ones(1,3) -1e0*ones(1,3)]]; % NOTE (Nathan): The 2* wasn't there last step
    auxdata.eventMax = [periodicityMult*[zeros(1,31)+0.5*pi/180 zeros(1,31)+1*pi/180 2*[10 25 10 10 25 10]] pathMult*[1e-1*ones(1,3) 1e0*ones(1,3)]]; % NOTE (Nathan): The 2* wasn't there last step
else
    auxdata.eventMin = [zeros(1,25)-0.5*pi/180 zeros(1,25)-1*pi/180 -2*[10 25 10 10 25 10], -2*[10 25 10 10 25 10] pathMult*[-1e-1*ones(1,3) -1e0*ones(1,3)]]; % NOTE (Nathan): The 2* wasn't there last step
    auxdata.eventMax = [zeros(1,25)+0.5*pi/180 zeros(1,25)+1*pi/180 2*[10 25 10 10 25 10], 2*[10 25 10 10 25 10] pathMult*[1e-1*ones(1,3) 1e0*ones(1,3)]]; % NOTE (Nathan): The 2* wasn't there last step
end


% Load Previous Solution
load('solution_taGasStimOpt_ipopt_v11pt12.mat');

% for iii = 0.8:0.1:tf
%     for jjj = 0:0.1:tf
% for iii = 0:0.1:tf
% paramScale = [1/0.7,1/0.7,1/tf,1/tf,1/1,1/1];
% auxdata.paramScale = paramScale;
auxdata.paramMin = zeros(1,6);
auxdata.paramMax = [0.7,0.7,tf,tf,1.6,1.6];

% load('optStimParams.mat');
% auxdata.optStimParams = optStimParams;
% auxdata.paramMin = [0,0,auxdata.optStimParams(3)-0.05,auxdata.optStimParams(4)-0.05];
% auxdata.paramMax = [0.7,0.7,auxdata.optStimParams(3)+0.05,auxdata.optStimParams(4)+0.05];

% auxdata.paramMin = [0,0,auxdata.gasStimTOn-0.05,auxdata.taStimTOn-0.05,0,0];
% auxdata.paramMax = [0.7,0.7,auxdata.gasStimTOn+0.05,auxdata.taStimTOn+0.05,1.6,1.6];
% auxdata.paramMin = [0,0,iii,jjj,0,0];
% auxdata.paramMax = [0.7,0.7,iii+0.1,jjj+0.1,1,1];
% auxdata.paramMin = [0,0,0,0];
% auxdata.paramMax = [0.7,0.7,1.6,1.6];
% auxdata.paramMin = [0,0,0.6,1.4,0,0];
% auxdata.paramMax = [0.7,0.7,0.7,1.5,1,1];
% auxdata.iii = iii;
% auxdata.paramMin = zeros(1,4);
% auxdata.paramMax = [0.7,0.7,tf,tf];

timeScale = tf*1;
auxdata.timeScale = timeScale;
paramScale = [0.7,0.7,0.1*timeScale*ones(1,4)];
% paramScale = [0.7,0.7,timeScale*ones(1,2)];
auxdata.paramScale = paramScale;

if 0 %Run Opt
%-------------------------------------------------------------------------%
%----------------------- Setup for Problem Bounds ------------------------%
%-------------------------------------------------------------------------%

nStates = length(auxdata.stateMin);
nControls = length(auxdata.controlMin);
% nparams = length(auxdata.paramsmin);
constrainInitConditions = false;

iphase = 1;
% bounds.phase(iphase).initialtime.lower = -.5;
% bounds.phase(iphase).initialtime.upper = -.5;
% bounds.phase(iphase).finaltime.lower = .5;
% bounds.phase(iphase).finaltime.upper = .5;
bounds.phase(iphase).initialtime.lower = t0/timeScale;
bounds.phase(iphase).initialtime.upper = t0/timeScale;
bounds.phase(iphase).finaltime.lower = tf/timeScale;
bounds.phase(iphase).finaltime.upper = tf/timeScale;
% states are the joint angles, velocities, accelerations and the joint
% torques
if constrainInitConditions
    bounds.phase(iphase).initialstate.lower = ([qGuess(1,:) qpGuess(1,:) qppGuess(1,:)]-10*pi/180 - (auxdata.stateMax+auxdata.stateMin)/2)./((auxdata.stateMax-auxdata.stateMin));
    bounds.phase(iphase).initialstate.upper = ([qGuess(1,:) qpGuess(1,:) qppGuess(1,:)]+10*pi/180 - (auxdata.stateMax+auxdata.stateMin)/2)./((auxdata.stateMax-auxdata.stateMin));
else
    bounds.phase(iphase).initialstate.lower = -0.5*ones(1,nStates);%,vmin];
    bounds.phase(iphase).initialstate.upper = 0.5*ones(1,nStates);
end
bounds.phase(iphase).finalstate.lower = -.5*ones(1,nStates);%,vmin];%[qf,qpf];
bounds.phase(iphase).finalstate.upper = .5*ones(1,nStates);%,vmax];%[qf,qpf];
bounds.phase(iphase).state.lower = -.5*ones(1,nStates);%,vmin];
bounds.phase(iphase).state.upper = .5*ones(1,nStates);%,vmax];
% Dynamics equations are forced to be satisfied by path constraints
% Control with joint angle jerk and the time derivative of the joint loads
bounds.phase(iphase).control.lower = -.5*ones(1,nControls);
bounds.phase(iphase).control.upper = .5*ones(1,nControls);
% bounds.phase(iphase).control.lower = [-0.5*ones(1,length(qpppMin)),-0.5*ones(1,10),-0.5*ones(1,length(handLoadMin))];
% bounds.phase(iphase).control.upper = [0.5*ones(1,length(qpppMax)),0.5*ones(1,10),0.5*ones(1,length(handLoadMax))];
bounds.phase(iphase).path.lower = -0.5*ones(size(auxdata.pathMin)); % Bounds on pelvis residuals
bounds.phase(iphase).path.upper = 0.5*ones(size(auxdata.pathMax));
bounds.phase(iphase).integral.lower = [0*(ones(1,length(auxdata.integralMin)-2)),-0.5,-0.5];
bounds.phase(iphase).integral.upper = [1*(ones(1,length(auxdata.integralMax)-2)),0.5,0.5];
% bounds.parameter.lower = -0.5*ones(size(auxdata.paramMin));
% bounds.parameter.upper = 0.5*ones(size(auxdata.paramMax));
bounds.parameter.lower = auxdata.paramMin./paramScale; %-0.5*ones(size(auxdata.paramMin));
bounds.parameter.upper = auxdata.paramMax./paramScale; %0.5*ones(size(auxdata.paramMax));


%-------------------------------------------------------------------------%
%---------------------- Provide Guess of Solution ------------------------%
%-------------------------------------------------------------------------%
% solution.phase.control(:,30:99) = solution.phase.control(:,30:99)/10;
% Set initial guess either from previous optimal control solution or from
% the auxilary data which was calculated from a different model
iphase = 1;
if 0
    guess = solution;
else
%     guess.phase(iphase).time    = ([auxdata.time]-(auxdata.tMax+auxdata.tMin)/2)./(auxdata.tMax-auxdata.tMin);
    guess.phase(iphase).time    = auxdata.time/timeScale;
    guess.phase(iphase).state   = ([qGuess qpGuess qppGuess]-onescol*(auxdata.stateMax+auxdata.stateMin)/2)./(onescol*(auxdata.stateMax-auxdata.stateMin));% auxdata.Loads];% auxdata.Loadsp]
    guess.phase(iphase).control = ([qpppGuess,auxdata.synergyCommands_r,auxdata.synergyCommands_l]-onescol*(auxdata.controlMax+auxdata.controlMin)/2)./(onescol*(auxdata.controlMax-auxdata.controlMin)); %zeros(size(auxdata.LoadsExp(:,:)));
    guess.parameter = [0.3,0.5,auxdata.gasStimTOn,auxdata.taStimTOn,0.06,0.6]./paramScale;
%     guess.parameter = [0.3,0.5,auxdata.gasStimTOn,auxdata.taStimTOn]./paramScale;
%     guess.parameter = [0.3,0.5,0.06,0.6]./paramScale; % Must be active for model calibration
    guess.phase(iphase).integral = (0.001 - (auxdata.integralMax+auxdata.integralMin)/2)./(auxdata.integralMax - auxdata.integralMin);
    
end


%-------------------------------------------------------------------------%
%------------- Set up Event Constraints That Link Phases -----------------%
%-------------------------------------------------------------------------%
bounds.eventgroup(1).lower = -0.5*ones(size(auxdata.eventMin));
bounds.eventgroup(1).upper = 0.5*ones(size(auxdata.eventMax));


%-------------------------------------------------------------------------%
%------------- Assemble Information into Problem Structure ---------------%
%-------------------------------------------------------------------------%
setup.name = 'openSim Full Body Motion';
setup.functions.continuous = @taGasStimOptContinuous;
auxdata.ContinuousFunc = setup.functions.continuous;
setup.functions.endpoint = @taGasStimOptEndpoint;
setup.auxdata = auxdata;
setup.bounds = bounds;
setup.guess = guess;
setup.nlp.solver = 'ipopt';
% setup.nlp.ipoptoptions.linear_solver = 'ma57';
setup.nlp.ipoptoptions.tolerance = 1e-8;
setup.nlp.snoptoptions.tolerance = 5e-5;
setup.nlp.ipoptoptions.maxiterations = 1000;
% setup.derivatives.stepsize1 = 1e-9;
setup.derivatives.supplier = 'sparseFD';
setup.derivatives.derivativelevel = 'first';
setup.derivatives.dependencies = 'sparse';
% setup.scales.method='automatic-bounds';
mesh.method = 'hp-PattersonRao';
mesh.tolerance       = 1e-4;
mesh.maxiterations    = 0;
mesh.colpointsmin    = 2;
mesh.colpointsmax    = 10;
N                    = 10; %10
mesh.phase.colpoints = 5*ones(1,N); %5
mesh.phase.fraction  = ones(1,N)/N;
setup.method = 'RPM-integration';
setup.mesh = mesh;

%-------------------------------------------------------------------------%
%------------------------- Solve Problem Using GPOP2 ---------------------%
%-------------------------------------------------------------------------%
tic
output = gpops2(setup);
toc
solution = output.result.solution;
% save(['solution_taGasStimOpt_ipopt_v5pt0_',strrep(num2str(iii),'.','p'),'x',strrep(num2str(jjj),'.','p'),'.mat'],'solution');
% save(['output_taGasStimOpt_ipopt_v5pt0_',strrep(num2str(iii),'.','p'),'x',strrep(num2str(jjj),'.','p'),'.mat'],'output');
% save(['solution_taGasStimOpt_ipopt_v9pt0_',strrep(num2str(iii),'.','p'),'.mat'],'solution');
% save(['output_taGasStimOpt_ipopt_v9pt0_',strrep(num2str(iii),'.','p'),'.mat'],'output');

save(['solution_taGasStimOpt_ipopt_v11pt14.mat'],'solution');
save(['output_taGasStimOpt_ipopt_v11pt14.mat'],'output');

end

%     end
% end
%-------------------------------------------------------------------------%
%------------------------------- Plot Solution ---------------------------%
%-------------------------------------------------------------------------%
input = solution;
input.auxdata = auxdata;
input.phase.parameter = solution.parameter;


outputContinuous = taGasStimOptContinuous(input);

onescol = ones(size(solution.phase.state(:,1)));

% solution.phase.time = (solution.phase.time)*(auxdata.tMax-auxdata.tMin)+(auxdata.tMax+auxdata.tMin)/2;
solution.phase.time = solution.phase.time*timeScale;
solution.phase.state = (solution.phase.state).*(onescol*(auxdata.stateMax-auxdata.stateMin))+onescol*(auxdata.stateMax+auxdata.stateMin)/2;
solution.phase.control = (solution.phase.control).*(onescol*(auxdata.controlMax-auxdata.controlMin))+onescol*(auxdata.controlMax+auxdata.controlMin)/2;
% solution.parameter = (solution.parameter).*((auxdata.paramMax-auxdata.paramMin))+(auxdata.paramMax+auxdata.paramMin)/2;
solution.parameter = solution.parameter.*paramScale;

% mkrDataSplined = interp1(auxdata.time,auxdata.mkrData,solution.phase.time,'pchip');
mkrDataSplined = fnval(auxdata.spMkrData,solution.phase.time)';

error = mkrDataSplined-outputContinuous.markers;

% Calculate RMS error for each marker at each time frame
for i = 1:size(mkrDataSplined,2)/3
    RMSError(:,i) = sum(error(:,(i-1)*3+1:3*i).^2,2).^(1/2);
end

% Calculate ground reactions about model midfoot location
mu_s = 0; % Coefficient of static friction for foot-ground contact
mu_d = 1;%input.phase.parameter(1,9);%input.auxdata.mu;%parameters(1,1); % Coefficient of dynamic friction
mu_v = 0; % Coefficient of viscous friction
latchvel = auxdata.latchvel;
kVal = auxdata.kVal;%input.auxdata.Kval; % Spring stiffnesses for foot ground contact
cVal = auxdata.cVal; % Spring damping
yVal = auxdata.yVal;
springMat = auxdata.springMat;
springBodyMat = auxdata.springBodyMat;
numSpringsBody = auxdata.numSpringsBody;
springMat(7:6+input.auxdata.numSprings/2,2) = springMat(7:6+input.auxdata.numSprings/2,2)+yVal(1);
springMat(7+input.auxdata.numSprings/2:end,2) = springMat(7+input.auxdata.numSprings/2:end,2)+yVal(2);


[springPos, springVels] = opensimPointKin_v3(solution.phase.time,solution.phase.state(:,1:62),springMat',springBodyMat);
springPos(:,1:18) = [];
springVels(:,1:18) = [];

footMkrNames = {'R.Midfoot.Superior' 'L.Midfoot.Superior'};
footMkrs = zeros(1,numel(footMkrNames));
for i = 1:numel(footMkrNames)
    footMkrs(i) = find(ismember(auxdata.mkrNames, footMkrNames{i}));
end

mcr = squeeze(outputContinuous.mkrPos(:,footMkrs(1),:));
mcl = squeeze(outputContinuous.mkrPos(:,footMkrs(2),:));
mcr(:,2) = 0;
mcl(:,2) = 0;
[footGRFsRMC, footGRFsLMC] = calcGroundReactions_v3(springPos, springVels, ...
    numSpringsBody, mu_s, mu_d, mu_v, kVal, cVal, mcr, mcl, latchvel, auxdata.tScale,auxdata.beltSpeed);


GRFsSplit = outputContinuous.GRFsSplit;
figure % 1
% plot(solution.phase.time/solution.phase.time(end),GRFsSplit(:,1:3)+GRFsSplit(:,4:6),'r')
plot(solution.phase.time/solution.phase.time(end),footGRFsRMC(:,1:3)+footGRFsRMC(:,4:6),'r')
hold on
plot(auxdata.time/auxdata.time(end),auxdata.grfDataR(:,1:3),'b')
title('Right GRFs');

figure % 2
% plot(solution.phase.time/solution.phase.time(end),GRFsSplit(:,13:15)+GRFsSplit(:,16:18),'r')
plot(solution.phase.time/solution.phase.time(end),footGRFsLMC(:,1:3)+footGRFsLMC(:,4:6),'r')
hold on
plot(auxdata.time/auxdata.time(end),auxdata.grfDataL(:,1:3),'b')
title('Left GRFs');

figure % 3
% plot(solution.phase.time/solution.phase.time(end),GRFsSplit(:,7:9)+GRFsSplit(:,10:12),'r')
plot(solution.phase.time/solution.phase.time(end),footGRFsRMC(:,7:9)+footGRFsRMC(:,10:12),'r')
hold on
plot(auxdata.time/auxdata.time(end),auxdata.grfDataR(:,4:6),'b')
title('Right GRMs');

figure % 4
% plot(solution.phase.time/solution.phase.time(end),GRFsSplit(:,19:21)+GRFsSplit(:,22:24),'r')
plot(solution.phase.time/solution.phase.time(end),footGRFsLMC(:,7:9)+footGRFsLMC(:,10:12),'r')
hold on
plot(auxdata.time/auxdata.time(end),auxdata.grfDataL(:,4:6),'b')
title('Left GRMs');

figure % 5
for i = 1:29
    if ismember(i,4:6)
        midpoint = ((max(qGuess(:,i)) - min(qGuess(:,i)))/2 + min(qGuess(:,i)));
        minpoint = midpoint - 0.1;
        maxpoint = midpoint + 0.1;
        subplot(5,6,i)
        plot(solution.phase.time/solution.phase.time(end), solution.phase.state(:,i),'r')
        hold on
    %     plot(auxdata.time/auxdata.time(end), auxdata.q(:,i),'b')
        plot(auxdata.time/auxdata.time(end), qGuess(:,i),'b')
        ylim([minpoint,maxpoint]);
        title(strrep(auxdata.qLabels(i),'_',' '));
    else
        midpoint = (180/pi)*((max(qGuess(:,i)) - min(qGuess(:,i)))/2 + min(qGuess(:,i)));
        minpoint = midpoint - 0.75*180/pi;
        maxpoint = midpoint + 0.75*180/pi;
        subplot(5,6,i)
        plot(solution.phase.time/solution.phase.time(end), solution.phase.state(:,i)*180/pi,'r')
        hold on
    %     plot(auxdata.time/auxdata.time(end), auxdata.q(:,i),'b')
        plot(auxdata.time/auxdata.time(end), qGuess(:,i)*180/pi,'b')
        ylim([minpoint,maxpoint]);
        title(strrep(auxdata.qLabels(i),'_',' '));
    end
end

figure % 6
for i = 1:29
    
    subplot(5,6,i)
    plot(solution.phase.time/solution.phase.time(end), solution.phase.control(:,i),'r')
    hold on
%     plot(auxdata.time/auxdata.time(end), auxdata.qpp(:,i),'b')
    plot(auxdata.time/auxdata.time(end), qpppGuess(:,i),'b')
    title(strrep(auxdata.qLabels(i),'_',' '));
end

figure % 7
for i = 1:23
    midpoint = ((max(auxdata.torques(:,i)) - min(auxdata.torques(:,i)))/2 + min(auxdata.torques(:,i)));
    minpoint = midpoint - 70;
    maxpoint = midpoint + 70;
    
    subplot(5,5,i)
    plot(solution.phase.time/solution.phase.time(end), outputContinuous.torques(:,i),'r')
    hold on
%     plot(auxdata.time/auxdata.time(end), auxdata.idData(:,i),'b')
    plot(auxdata.time/auxdata.time(end), auxdata.torques(:,i),'b')
    ylim([minpoint,maxpoint]);
    title(strrep(auxdata.torqueNames(i),'_',' '));
end


neuralCommands = solution.phase.control(:,32:41);
neuralCommands_r = neuralCommands(:,1:5);
neuralCommands_l = neuralCommands(:,6:10);
synergyWeights_r = auxdata.synergyWeights_r;
synergyWeights_l = auxdata.synergyWeights_l; %reshape(input.phase.parameter(1,:),10,35);
synergyTime = solution.phase.time;

% activations(:,1:36) = neuralCommands(:,1:5)*synergyWeights_r;
% activations(:,37:72) = neuralCommands(:,6:10)*synergyWeights_l;
activations = outputContinuous.actsWithStim;

figure %10
title('Activation Matching')
for i = 1:36
    subplot(6,6,i)
    plot(solution.phase.time/solution.phase.time(end), activations(:,i),'r')
    hold on
    plot(auxdata.time/auxdata.time(end), auxdata.acts(1:101,i),'b')
    axis([0 1 0 1])
    title(strrep(auxdata.muscleNames(i),'_',' '));
end

figure %11
title('Activation Matching')
for i = 1:36
    subplot(6,6,i)
    plot(solution.phase.time/solution.phase.time(end), activations(:,i+36),'r')
    hold on
    plot(auxdata.time/auxdata.time(end), auxdata.acts(1:101,i+36),'b')
    axis([0 1 0 1])
    title(strrep(auxdata.muscleNames(i+36),'_',' '));
end

figure %12
title('Synergy Commands (R)')
for iii = 1:5
    subplot(2,3,iii)
    plot(solution.phase.time/solution.phase.time(end),neuralCommands(:,iii),'r');
    hold on
    plot(auxdata.time/auxdata.time(end),auxdata.synergyCommands_r(:,iii),'b');
end

figure %13
title('Synergy Commands (L)')
for iii = 1:5
    subplot(2,3,iii)
    plot(solution.phase.time/solution.phase.time(end),neuralCommands(:,iii+5),'r');
    hold on
    plot(auxdata.time/auxdata.time(end),auxdata.synergyCommands_l(:,iii),'b');
end


figure %14
subplot(1,2,1)
plot(solution.phase.time/solution.phase.time(end),footGRFsLMC(:,1)+footGRFsLMC(:,4),'r')
hold on
plot(auxdata.time/auxdata.time(end),auxdata.grfDataL(:,1),'b')
title('Left Propulsive Force');
subplot(1,2,2)
plot(solution.phase.time/solution.phase.time(end),footGRFsRMC(:,1)+footGRFsRMC(:,4),'r')
hold on
plot(auxdata.time/auxdata.time(end),auxdata.grfDataR(:,1),'b')
title('Right Propulsive Force');


apForceR = GRFsSplit(:,1)+GRFsSplit(:,4);
apForceL = GRFsSplit(:,13)+GRFsSplit(:,16);
propulsiveForceR = apForceR.*(tanh(apForceR) + 1)/2;
propulsiveForceL = apForceL.*(tanh(apForceL) + 1)/2;

apForceR_integral = trapz(solution.phase.time,apForceR)
apForceL_integral = trapz(solution.phase.time,apForceL)
propulsiveForceR_integral = trapz(solution.phase.time,propulsiveForceR)
propulsiveForceL_integral = trapz(solution.phase.time,propulsiveForceL)


%% Output Kinematics for OpenSim
outputIKMot('taGasStimOpt_Trial04_Cycle12.mot',solution.phase.state(:,1:31),solution.phase.time);
